/**
 * Copyright (c) 1993 Richard V. Nash.
 * Copyright (c) 2000 Dan Papasian.  All rights reserved.
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _GAMEDB_H
#define _GAMEDB_H

extern const char *bstr[];
extern const char *rstr[];

#define GAMEFILE_VERSION 5 
#define MAX_GLINE_SIZE 1024

#define REL_GAME 0
#define REL_SPOS 1
#define REL_REFRESH 2
#define REL_EXAMINE 3

enum gamestatus {GAME_EMPTY, GAME_NEW, GAME_ACTIVE, GAME_EXAMINE, GAME_SETUP};

/* Do not change the order of these - DAV */
enum gametype {TYPE_UNTIMED, TYPE_BLITZ, TYPE_STAND, TYPE_NONSTANDARD,
               TYPE_WILD, TYPE_LIGHT, TYPE_BUGHOUSE};

#define NUM_GAMETYPES 7 

/* OK, DAV, I'll try it another way. -- hersco */
enum ratetype {RATE_STAND, RATE_BLITZ, RATE_WILD, RATE_LIGHT, RATE_BUGHOUSE};
#define NUM_RATEDTYPE 5

#define FLAG_CHECKING -1
#define FLAG_NONE 0
#define FLAG_CALLED 1
#define FLAG_ABORT 2

enum {
	END_CHECKMATE = 0,
	END_RESIGN,
	END_FLAG,
	END_AGREEDDRAW,
	END_REPETITION,
	END_50MOVERULE,
	END_ADJOURN,
	END_LOSTCONNECTION,
	END_ABORT,
	END_STALEMATE,
	END_NOTENDED,
	END_COURTESY,
	END_BOTHFLAG,
	END_NOMATERIAL,
	END_FLAGNOMATERIAL,
	END_ADJDRAW,
	END_ADJWIN,
	END_ADJABORT,
	END_COURTESYADJOURN
};

typedef struct journal {
  char slot;
  char WhiteName[MAX_LOGIN_NAME];
  int WhiteRating;
  char BlackName[MAX_LOGIN_NAME];
  int BlackRating;
  char type[4];
  int t;
  int i;
  char eco[4];
  char ending[4];
  char result[8];
} journal;


typedef struct game {
  /* Saved in the game file */
  int wInitTime, wIncrement;
  int bInitTime, bIncrement;
  time_t timeOfStart;
  int wLastRealTime;
  int wRealTime;
  int wTimeWhenReceivedMove;
  int wTimeWhenMoved;
  int bLastRealTime;
  int bRealTime;
  int bTimeWhenReceivedMove;
  int bTimeWhenMoved;
  int flag_pending;
  unsigned long flag_check_time;
  int wTime;
  int bTime;
  int clockStopped;
  int rated;
  int private;
  enum gametype type;
  int passes; /* For simul's */
  int numHalfMoves;
  move_t *moveList;       /* primary movelist */
  char FENstartPos[74];    /* Save the starting position. */
  game_state_t game_state;

  char white_name[MAX_LOGIN_NAME];    /* to hold the playername even after he disconnects */        
  char black_name[MAX_LOGIN_NAME];    
  int white_rating;
  int black_rating;

  /* Not saved in game file */
  int revertHalfMove;
  int totalHalfMoves;
  int white;
  int black;
  int link;
  enum gamestatus status;
  int moveListSize; /* Total allocated in *moveList */
  int examHalfMoves;
  move_t *examMoveList;     /* extra movelist for examine */
  int examMoveListSize;  

  unsigned startTime;    /* The relative time the game started  */
  unsigned lastMoveTime; /* Last time a move was made */
  unsigned lastDecTime;  /* Last time a players clock was decremented */

  int result;
  int winner;

} game;

extern const char *TypeStrings[NUM_GAMETYPES];
extern const char *TypeChars[NUM_GAMETYPES];

#endif

